# clasificador_urbanizacion.py
from modulos.clasificadores import clasificar_urbanizacion
from modulos.clasificador.clasificador_base import ClasificadorBase

class ClasificadorUrbanizacion(ClasificadorBase):
    def clasificar(self):
        return clasificar_urbanizacion(self.rutas_indices['NDBI'],
                                       self.rutas_indices['NDBAI'],
                                       self.rutas_indices['NDMI'],
                                       self.rutas_indices['NDVI'])


# procesador_imagenes.py
import os
from modulos.crop_tif import crop_tif

class RecortadorImagenes:
    def __init__(self, carpeta_recortadas, carpeta_no_recortadas, ruta_poligono):
        self.carpeta_recortadas = carpeta_recortadas
        self.carpeta_no_recortadas = carpeta_no_recortadas
        self.ruta_poligono = ruta_poligono

    def recortar_imagenes(self):
        if os.path.exists(self.carpeta_recortadas):
            print(f"La carpeta '{self.carpeta_recortadas}' existe.")
            os.system(f"rm -rf {self.carpeta_recortadas}/*")
        else:
            print(f"La carpeta '{self.carpeta_recortadas}' no existe. Se creará ahora.")
            try:
                os.makedirs(self.carpeta_recortadas)
                print(f"Se ha creado la carpeta '{self.carpeta_recortadas}/'.")
            except OSError as e:
                print(f"No se pudo crear la carpeta '{self.carpeta_recortadas}': {e}")

        for nombre_archivo in os.listdir(self.carpeta_no_recortadas):
            if nombre_archivo.endswith(".TIF") or nombre_archivo.endswith(".tif"):
                ruta_completa = os.path.join(self.carpeta_no_recortadas, nombre_archivo)
                crop_tif(ruta_completa, f"{self.carpeta_recortadas}/{nombre_archivo}", self.ruta_poligono)


import subprocess

class ManejadorRutasBandas:
    def __init__(self, carpeta_recortadas):
        self.carpeta_recortadas = carpeta_recortadas.replace(" ", "\ ")
        self.rutas = {
            'B4': None,
            'B5': None,
            'B3': None,
            'B6': None,
            'B10': None,
            'NDVI': f"{self.carpeta_recortadas}/ndvi.TIF",
            'NDWI': f"{self.carpeta_recortadas}/ndwi.TIF",
            'NDBI': f"{self.carpeta_recortadas}/ndbi.TIF",
            'NDBAI': f"{self.carpeta_recortadas}/ndbai.TIF",
            'NDMI': f"{self.carpeta_recortadas}/ndmi.TIF"
        }
        self.cargar_rutas()

    def cargar_rutas(self):
        bandas = ["B4", "B5", "B3", "B6", "B10"]
        for banda in bandas:
            ruta = subprocess.run(f"ls {self.carpeta_recortadas}/*{banda}.TIF",
                                  shell=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
                                  text=True,
                                  check=False).stdout.strip("\n")
            self.rutas[banda] = ruta

    def obtener_ruta(self, banda):
        return self.rutas.get(banda)

    def establecer_ruta(self, banda, ruta):
        self.rutas[banda] = ruta


# clasificador_suelos.py
import rasterio
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap
from modulos.clasificadores import clasificar_clases
from modulos.clasificador.clasificador_base import ClasificadorBase

class ClasificadorSuelos(ClasificadorBase):
    def __init__(self, carpeta_recortadas, rutas_indices):
        super().__init__(rutas_indices)
        self.carpeta_recortadas = carpeta_recortadas

    def clasificar(self):
        ARCHIVO_CLASES = f"{self.carpeta_recortadas}/class.TIF"
        clasificar_clases(self.rutas_indices['NDVI'], self.rutas_indices['NDWI'], ARCHIVO_CLASES)

        with rasterio.open(ARCHIVO_CLASES) as src:
            clasificacion = src.read(1, masked=True)
            profile = src.profile

        colors = ['brown', 'green', 'yellow', 'blue']
        cmap = ListedColormap(colors)
        fig, ax = plt.subplots(figsize=(10, 10))
        im = ax.imshow(clasificacion, cmap=cmap, vmin=1, vmax=4, interpolation='none')
        cbar = ax.figure.colorbar(im, ax=ax, ticks=[1, 2, 3, 4], orientation='vertical')
        cbar.ax.set_yticklabels(['Suelo expuesto', 'Vegetación baja', 'Vegetación alta', 'Agua'])
        cbar.set_label('Clases')
        ax.set_title('Clasificación de Imagen')
        plt.show()
        print("\nFinaliza clasificación de clases.")


# clasificador_base.py
class ClasificadorBase:
    def __init__(self, rutas_indices):
        self.rutas_indices = rutas_indices

    def clasificar(self):
        raise NotImplementedError("Subclasses should implement this method")


from modulos.calculo_indices import calcular_ndvi, calcular_ndwi, calcular_ndbi, calcular_ndbai

class CalculadorIndices:
    def __init__(self, manejador_rutas):
        self.manejador_rutas = manejador_rutas

    def calcular_indices(self):
        calcular_ndvi(self.manejador_rutas.obtener_ruta('B4'),
                      self.manejador_rutas.obtener_ruta('B5'),
                      self.manejador_rutas.obtener_ruta('NDVI'))
        print("\t- Se calculó el NDVI")

        calcular_ndwi(self.manejador_rutas.obtener_ruta('B3'),
                      self.manejador_rutas.obtener_ruta('B5'),
                      self.manejador_rutas.obtener_ruta('NDWI'))
        print("\t- Se calculó el NDWI")

        calcular_ndbi(self.manejador_rutas.obtener_ruta('B5'),
                      self.manejador_rutas.obtener_ruta('B6'),
                      self.manejador_rutas.obtener_ruta('NDBI'))
        print("\t- Se calculó el NDBI")

        calcular_ndbai(self.manejador_rutas.obtener_ruta('B6'),
                       self.manejador_rutas.obtener_ruta('B10'),
                       self.manejador_rutas.obtener_ruta('NDBAI'))
        print("\t- Se calculó el NDBaI")

        calcular_ndbi(self.manejador_rutas.obtener_ruta('B5'),
                      self.manejador_rutas.obtener_ruta('B6'),
                      self.manejador_rutas.obtener_ruta('NDMI'))
        print("\t- Se calculó el NDMI")